/*
 * config.c
 */

__asm("	text\n	dc.b	'$Id: config.c,v 1.8 1998/06/09 13:00:38 sharl Exp $'\n	even\n");

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <err.h>
#include <sys/xglob.h>
#include <sys/stat.h>

#include "config.h"
#include "util.h"
#include "ircdef.h"

char *irc_server = NULL;
char *irc_pass = NULL;
int irc_port = DEFAULT_PORT;
char *irc_nick = NULL;
char *irc_user = NULL;
char *irc_name = NULL;
char *irc_info = NULL;
char *irc_quit_message = NULL;
time_t irc_timeout = DEFAULT_TIMEOUT;
char *irc_keyword = NULL;
char *irc_keyword_bell = NULL;
char *irc_keyword_bold = NULL;
char *irc_autochanops = NULL;
char **irc_autochanop = NULL;
char *irc_autonicks = NULL;
char **irc_autonick = NULL;
char *irc_channel[IRC_JOIN_MAX + 1] = {NULL};

static char *irc_channels = NULL;

char *keyword_bell = NULL;	/* KEYWORD p ADPCM obt@ */
size_t bell_length;			/* ADPCM obt@ */

DIREC directive[] = {
	{RC_AUTONICK,		STR,	&irc_autonicks},
	{RC_AUTOCHANOP,		STR,	&irc_autochanops},
	{RC_CHANNEL,		STR,	&irc_channels},
//	{RC_IGNORE,			STR,	&irc_ignores},
	{RC_INFO,			STR,	&irc_info},
//	{RC_ISON,			STR,	&irc_isons},
//	{RC_ISON_INTERVAL,	NUM,	&irc_ison_interval},
//	{RC_LOG_CHANNEL,	STR,	&irc_log_channels},
//	{RC_LOG_DIRECTORY,	STR,	&irc_log_directory},
//	{RC_LOG_FORMAT,		STR,	&irc_log_format},
	{RC_NICK,			STR,	&irc_nick},
	{RC_NAME,			STR,	&irc_name},
	{RC_KEYWORD,		STR,	&irc_keyword},
	{RC_KEYWORD_BELL,	STR,	&irc_keyword_bell},
	{RC_KEYWORD_BOLD,	STR,	&irc_keyword_bold},
	{RC_PASS,			STR,	&irc_pass},
	{RC_PORT,			NUM,	&irc_port},
	{RC_QUIT,			STR,	&irc_quit_message},
	{RC_SERVER,			STR,	&irc_server},
	{RC_TIMEOUT,		NUM,	&irc_timeout},
	{RC_USER,			STR,	&irc_user},
	{NULL},
};

/****************************************
****************************************/
static char *
skipspace(char *line)
{
	char *p;

	for (p = line; *p; p++) {
		if (*p != ' ' && *p != '\t')
			break;
	}
	return p;
}

/****************************************
****************************************/
static void
choplf(char *buf)
{
	if (*(buf + strlen(buf) - 1) == '\n')
		*(buf + strlen(buf) - 1) = '\0';
}

/****************************************
****************************************/
static void
getword(char *keyword, char *value, char *line)
{
	line = skipspace(line);
	strtok(line, " \t\n");
	strcpy(keyword, line);
	line = skipspace(line + strlen(keyword) + 1);
	strcpy(value, line);
}

/****************************************
	channel 
	0 ŐI
	-1  IRC_JOIN_MAX z
****************************************/
static int
div_channels()
{
	int count;
	int inflag = 0;
	char *p;

	for (count = 0; count < IRC_JOIN_MAX; count++)
		irc_channel[count] = NULL;
	if (irc_channels) {
		count = 0;
		for (p = irc_channels; *p; p++) {
			if (inflag == 0 && *p != ' ' && *p != '\t') {
				if (count >= IRC_JOIN_MAX)
					return -1;
				irc_channel[count++] = p;
				inflag = 1;
			}
			if (*p == ' ' || *p == '\t') {
				inflag = 0;
			}
		}
		irc_channel[count] = NULL;
	}

	for (count = 0; count < IRC_JOIN_MAX && irc_channel[count]; count++)
		for (p = irc_channel[count]; *p; p++)
			if (*p == ' ' || *p == '\t')
				*p = '\0';
	return 0;
}

/****************************************
	󔒂ŋ؂Ĕz֊i[
****************************************/
void
div_string(char *line, char ***array)
{
	char **ptr;

	*array = (char **)xmalloc(strlen(line) * sizeof(char *) + 1);
	if (*array == NULL) {
		fprintf(stderr, "ex\n");
		return;
	}
	ptr = *array;

	while (*line) {
		char *p;
		*ptr = xmalloc(strlen(line) + 1);
		strcpy(*ptr, line);
		for (p = *ptr++; *p; p++)
			if (*p == ' ' || *p == '\t')
				*p = '\0';
		while (*line && *line != ' ' && *line != '\t')
			line++;
		line++;
	}
	*ptr = NULL;
}

/****************************************
	configuration file ǂݍ
	I: 0
	ُI: -1
****************************************/
int
ReadConfig()
{
	FILE *fp;
	char *hdir, buf[BUFSIZ], keyword[BUFSIZ], value[BUFSIZ];

	if ((hdir = getenv("HOME")) == NULL)
		hdir = ".";
	hdir = _addlastsep(hdir);
	sprintf(buf, "%s" DEFAULT_RC, hdir);
	fp = fopen(buf, "r");
	if (fp == NULL) {
		warn("%s", buf);
		sprintf(buf, "%s" DEFAULT_RC2, hdir);
	}
	if (fp == NULL) {
		warn("%s", buf);
		return -1;
	}
	while (fgets(buf, sizeof buf, fp) != NULL) {
		DIREC *dir;

		choplf(buf);
		if (*buf == '#' || *buf == '\0')
			continue;
		getword(keyword, value, buf);
		if (*buf == '\0')
			continue;

		for (dir = directive; dir->name; dir++) {
			if (tolower(*keyword) == tolower(*dir->name)
				&& !strcasecmp(keyword + 1, dir->name + 1)) {
				switch (dir->type) {
					case STR:
						*(char **)(dir->var) = xmalloc(strlen(value) + 1);
						strcpy(*(char **)(dir->var), value);
						break;
					case NUM:
						*(long *)(dir->var) = atoi(value);
						break;
					default:
						warnx("ReadCondfig: internal error.\n");
						break;
				}
			}
		}
	}
	fclose(fp);

	/* channel  */
	if (div_channels() < 0)
		errx(EXIT_FAILURE, "CHANNEL is over %d.", IRC_JOIN_MAX);
	if (irc_autonicks)
		div_string(irc_autonicks, &irc_autonick);
	if (irc_autochanops)
		div_string(irc_autochanops, &irc_autochanop);

	if (irc_keyword_bell) {
		FILE *fp;
		struct stat st;

		if ((fp = fopen(irc_keyword_bell, "rb")) == NULL) {
			perror(irc_keyword_bell);
			return -1;
		}
		fstat(fileno(fp), &st);
		bell_length = st.st_size;
		keyword_bell = xmalloc(bell_length);
		fread(keyword_bell, sizeof(char), bell_length, fp);
		fclose(fp);
	}

	return 0;
}
