/*
 * console.c
 */

__asm("	text\n	dc.b	'$Id: console.c,v 1.11 1998/06/09 13:01:30 sharl Exp $'\n	even\n");

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <conio.h>
#include <mbctype.h>
#include <sys/socket.h>
#define __IOCS_INLINE__
#include <iocslib.h>
#define __DOS_INLINE__
#include <doslib.h>

#include "config.h"
#include "util.h"
#include "console.h"
#include "command.h"
#include "channel.h"
#include "ircdef.h"
#include "kanji.h"

#define MES			"================================================================================================"
#define	BDCOLOR		(0x0000)
#define	FGCOLOR		(0xFFFE)

#define TIMEOUT_APPEAR	(180)
#define TIMEOUT_COLUMN	(2)
#define CHANNEL_COLUMN	(8)

time_t timeout;		/* active timeout */
static char modeline[] = MES;

static int oldbrk;
static BOOLEAN ctrlx = FALSE;

/****************************************
	J[R[h𓾂
****************************************/
static int
getpcode(char *ep)
{
	int r, g, b;

	if (sscanf(ep, " %u %u %u", &r, &g, &b) != 3)
		return 0;
	return (g << 11) | (r << 6) | (b << 1);
}

/****************************************
	J[ݒ
****************************************/
static void
setcolortable()
{
	char *ep;

	ep = getenv("BDCOLOR");
	_iocs_tpalet(0, ep ? getpcode(ep) : BDCOLOR);
	ep = getenv("FGCOLOR");
	_iocs_tpalet(3, ep ? getpcode(ep) : FGCOLOR);
}

/****************************************
	J[\`
****************************************/
static void
curput(int x, int y)
{
	int i;
	int plane;
	static struct _fntbuf cursor = {
		8, 16,
	};

	if (x > CONSOLE_WIDTH - 1)
		return;
	for (plane = 1; plane < 3; plane++) {
		_iocs_tcolor(plane);
		_iocs_textget(x * 8, y * 16, &cursor);
		for (i = 0; i < cursor.yl; i++) {
			cursor.buffer[i] ^= 0xff;
		}
		_iocs_textput(x * 8, y * 16, &cursor);
	}
	_iocs_tcolor(0);
}

/****************************************
	modeline update
****************************************/
static void
mdupdate()
{
	char buf[256];
	time_t t;

	strcpy(modeline, MES);
	t = timeout + irc_timeout - time(NULL);
	if (t < TIMEOUT_APPEAR) {
		sprintf(buf, " %d ", t);
		memcpy(modeline + TIMEOUT_COLUMN, buf, strlen(buf));
	}
	if (current_channel >= 0 && chanlist[current_channel].active != FALSE) {
		CHANSTAT *ccsp = &chanlist[current_channel];
		char *p = modeline + CHANNEL_COLUMN;

		*p++ = ' ';
		memcpy(p, ccsp->name, strlen(ccsp->name));
		p += strlen(ccsp->name);
		*p = ' ';
		if (ccsp->topic) {
			p += 3;
			*p++ = ' ';
			memcpy(p, ccsp->topic, strlen(ccsp->topic));
			p += strlen(ccsp->topic);
			*p = ' ';
		}
	}

	_iocs_b_putmes(13, 0, CONSOLE_HEIGHT, CONSOLE_WIDTH - 1, modeline);
}

/****************************************
	s͕ɕ\
****************************************/
static void
mdset(char *mes)
{
	_iocs_b_putmes(3, 0, CONSOLE_INPUT, CONSOLE_WIDTH - 1, mes);
}

/****************************************
	ʏ
****************************************/
void
init_console()
{
	_iocs_crtmod(0x10);
	setcolortable();
	_iocs_b_consol(0, 0, CONSOLE_WIDTH - 1, CONSOLE_HEIGHT - 1);
	_iocs_b_putmes(13, 0, CONSOLE_HEIGHT, CONSOLE_WIDTH - 1, modeline);
	_dos_c_curoff();
	oldbrk = _dos_breakck(2);
	curput(0, CONSOLE_INPUT);
	printf("%s \033[31mconnected to %s:%d\033[m\n"
		   "%s \033[32mactive timeout = %d second\033[m\n",
		   getnow(), irc_server, irc_port, getnow(), irc_timeout);
}

/****************************************
	ʂ߂
****************************************/
void
tini_console()
{
	_iocs_b_consol(0, 0, CONSOLE_WIDTH - 1, CONSOLE_OHEIGHT - 1);
	_iocs_b_locate(0, CONSOLE_OHEIGHT - 1);
	_iocs_tpalet(0, BDCOLOR);
	_iocs_tpalet(3, FGCOLOR);
	_dos_c_curon();
	_dos_breakck(oldbrk);
}

static char onelinebuf[1024];

/****************************************
	L[́EsҏW
****************************************/
BOOLEAN
irc_console(int sock)
{
	static char *p = onelinebuf;
	static int count = 0;
	char linebuf[1024];
	char newbuf[1024];
	char chanbuf[1024];

	if (kbhit()) {
		int c;

		timeout = time(NULL);
		c = getch();
		if (ismbblead(c)) {
			*p++ = c;
			*p++ = getch();
			count += 2;
			*p = '\0';
		} else if (c == 0x02) {		/* CTRL+B */
			if (ctrlx != FALSE) {
				ctrlx = FALSE;
				list_joined();
			}
		} else if (c == 0x03) {		/* CTRL+C */
			if (ctrlx == FALSE) {
				char tmpbuf[BUFSIZ];

				while (socklen(sock, 0))
					(void)recvline(sock, tmpbuf, BUFSIZ - 1);
			} else {
				mdset("");
				irc_quit(sock, irc_quit_message);
				return FALSE;
			}
		} else if (c == 0x08) {		/* CTRL+H, BS */
			if (count == 0)
				return TRUE;
			*--p = '\0';
			--count;
			if (ismbblead(*(p - 1))) {
				*--p = '\0';
				--count;
			}
		} else if (c == 0x09) {		/* TAB, CTRL+I */
			if (current_channel >= 0) {
				new_channel(current_channel);
				mdupdate();
			}
		} else if (c == '\r') {		/* CTRL+M */
			*p = '\0';	/* 󃊃^[h~ */
			mdset("");
			if (*onelinebuf != '/' && *onelinebuf) {
				if (current_channel >= 0) {
					printf("%s >%s:%s< %s\n",
						   getnow(), chanlist[current_channel].name, irc_nick, onelinebuf);
					sjstr2jstr(newbuf, onelinebuf);
					sjstr2jstr(chanbuf, chanlist[current_channel].name);
					sprintf(linebuf, "PRIVMSG %s :%s\r\n", chanbuf, newbuf);
#ifdef DEBUG
					if (debug)
						printf(linebuf);
#endif
					write(sock, linebuf, strlen(linebuf));
				}
			} else if (*onelinebuf == '/') {
				irc_command(sock, onelinebuf + 1);
			}
			*(p = onelinebuf) = '\0';
			count = 0;
		} else if (c == 0x0E) {		/* CTRL+N */
			if (ctrlx != FALSE) {
				ctrlx = FALSE;
				irc_names(sock);
			}
		} else if (c == 0x15) {		/* CTRL+U */
			*(p = onelinebuf) = '\0';
			count = 0;
		} else if (c == 0x18) {		/* CTRL+X */
			if (ctrlx == FALSE) {
				ctrlx = TRUE;
				mdset("[CTRL-X]");
			} else {
				ctrlx = FALSE;
				mdset("");
			}
			return TRUE;
		} else {
			if (ctrlx == TRUE) {
				ctrlx = FALSE;
				mdset("");
				return TRUE;
			}
			if (c < ' ')
				return TRUE;
			*p++ = c;
			count++;
			*p = '\0';
		}
		if (count >= CONSOLE_WIDTH) {
			count = CONSOLE_WIDTH - 1;
			p = onelinebuf + count;
			*p = '\0';
			if (ismbblead(*(p - 1))) {
				*--p = '\0';
				--count;
			}
			putchar('\007');
			fflush(stdout);
		}
		_iocs_b_putmes(3, 0, CONSOLE_INPUT, CONSOLE_WIDTH - 1, onelinebuf);
		curput(count, CONSOLE_INPUT);
	} else
		mdupdate();
	return TRUE;
}
