/*
 * irc.c
 */

__asm("	text\n	dc.b	'$Id: irc.c,v 1.11 1998/06/09 13:02:02 sharl Exp $'\n	even\n");

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#define __IOCS_INLINE__
#include <iocslib.h>

#include "config.h"
#include "console.h"
#include "ircdef.h"
#include "irc.h"
#include "command.h"
#include "channel.h"
#include "util.h"
#include "kanji.h"

/****************************************
	T[õbZ[W
****************************************/
BOOLEAN
irc_fromserver(char *buf)
{
	char newbuf[1024], tmpbuf[1024];
	char chan[512], chanbuf[512];
	int num, len;

	if (sscanf(buf, "%*s %d %*s %n", &num, &len)) {
#ifdef DEBUG
		if (debug)
			printf(buf);
#endif
		switch (num){
			case 1: case 2: case 3:
			case RPL_LUSERCLIENT:
			case RPL_LUSERME:
			case RPL_MOTDSTART:
			case RPL_UNAWAY:
			case RPL_NOWAWAY:
				printf("%s \033[31m%s\033[m", getnow(), buf + len + 1);
				break;
			case RPL_INFO:
			case RPL_MOTD:
				jstr2sjstr(newbuf, buf + len + 1);
				printf("%s \033[31m%s\033[m", getnow(), newbuf);
				break;
			case 4:
			case RPL_LUSERUNKNOWN:
			case RPL_LUSERCHANNELS:
				printf("%s \033[31m%s\033[m", getnow(), buf + len);
				break;
			case RPL_INVITING:
				jstr2sjstr(newbuf, buf + len);
				printf("%s \033[31mINVITE %s\033[m", getnow(), newbuf);
				break;
			case RPL_TOPIC:
				{
					int n;
					char *p;

					sscanf(buf + len, "%s :%n", chan, &n);
					jstr2sjstr(chanbuf, chan);
					p = tmpbuf;
					buf += len + n;
					while (*buf && *buf != '\r')
						*p++ = *buf++;
					*p = '\0';
					jstr2sjstr(newbuf, tmpbuf);
					printf("%s \033[31mTOPIC %s %s\033[m\n", getnow(), chanbuf, newbuf);
					set_topic(chanbuf, newbuf);
				}
				break;
			case RPL_VERSION:
				printf("%s \033[31mVERSION %s\033[m", getnow(), buf + len);
				break;
			case RPL_ENDOFINFO:
			case RPL_ENDOFMOTD:
			case ERR_NOTEXTTOSEND:
				break;
			case ERR_NICKNAMEINUSE:
				printf("%s \033[32m%s\033[m", getnow(), buf + len);
				return FALSE;
			case ERR_UNKNOWNCOMMAND:
				jstr2sjstr(newbuf, buf + len);
				printf("\033[36m%s\033[m", newbuf);
				break;
			default:
				jstr2sjstr(newbuf, buf + len);
				printf("%s \033[36m%d\033[m %s", getnow(), num, newbuf);
				break;
		}
	} else {
		printf("%s \033[36m%s\033[m", getnow(), buf);
	}
	return TRUE;
}

/****************************************
	[ŨbZ[W
****************************************/
void
irc_fromuser(char *buf)
{
	char newbuf[1024];
	char from[512];
	char comm[512];
	char to[512];
	int len;

	sscanf(buf, "%s %s %s %n", from, comm, to, &len);
	if (!strcmp(from, irc_nick) && !strcmp(to, irc_nick) && !strcmp(comm, "MODE")) {
		printf("%s Your MODE is set %s", getnow(), buf + len);
	} else {
		jstr2sjstr(newbuf, buf);
		printf("%s \033[36m%s\033[m", getnow(), newbuf);
	}
}

/****************************************
	CTCP 
****************************************/
static void
irc_ctcpfrom(int sock, char *nick, char *buf)
{
	char line[1024];
	char newbuf[1024];

	if (!strncasecmp(buf, "\001PING", 5)) {
		time_t ping;

		if (sscanf(buf + 1, "%*s %d\001", &ping) != EOF) {
			printf("%s \033[31mCTCP PING from %s\033[m\n", getnow(), nick);
			sprintf(line, "NOTICE %s :\001PING %d\001\r\n", nick, ping);
#ifdef DEBUG
			if (debug)
				printf(line);
#endif
			write(sock, line, strlen(line));
		}
	} else if (!strcasecmp(buf, "\001TIME\001")) {
		time_t tim;
		struct tm *tm;
		char timbuf[256];

		tim = time(NULL);
		tm = localtime(&tim);
		printf("%s \033[31mCTCP TIME from %s\033[m\n", getnow(), nick);
		strftime(timbuf, sizeof(timbuf), "%a %b %d %T %Y", tm);
		sprintf(line, "NOTICE %s :\001TIME %s\001\r\n", nick, timbuf);
#ifdef DEBUG
		if (debug)
			printf(line);
#endif
		write(sock, line, strlen(line));
	} else if (!strcasecmp(buf, "\001VERSION\001")) {
		printf("%s \033[31mCTCP VERSION from %s\033[m\n", getnow(), nick);
		sprintf(line,
				"NOTICE %s :\001VERSION " PROGNAME " " VERSION " (X68k)\001\r\n",
				nick);
		sjstr2jstr(newbuf, line);
#ifdef DEBUG
		if (debug)
			printf(line);
#endif
		write(sock, newbuf, strlen(newbuf));
	} else if (!strcasecmp(buf, "\001USERINFO\001")) {
		printf("%s \033[31mCTCP USERINFO from %s\033[m\n", getnow(), nick);
		sprintf(line, "NOTICE %s :\001USERINFO :%s\001\r\n", nick, irc_info);
		sjstr2jstr(newbuf, line);
#ifdef DEBUG
		if (debug)
			printf(line);
#endif
		write(sock, newbuf, strlen(newbuf));
	} else if (!strcmp(buf, "\001CLIENTINFO\001")) {
		printf("%s \033[31mCTCP CLIENTINFO from %s\033[m\n", getnow(), nick);
		sprintf(line,
				"NOTICE %s :\001CLIENTINFO :VERSION CLIENTINFO USERINFO PING TIME\001\r\n",
				nick);
#ifdef DEBUG
		if (debug)
			printf(line);
#endif
		write(sock, line, strlen(line));
	}
}

/****************************************
	s
****************************************/
BOOLEAN
irc_parseline(int sock, char *buf)
{
	char newbuf[1024];
	char chanbuf[1024];
	char nick[512] = {0};
	char chan[512] = {0};
	char comm[512] = {0};
	char rest[512] = {0};

	if (*buf == ':') {
		if (!strncmp(buf + 1, irc_server, strlen(irc_server))) {	/* T[õbZ[W */
			return irc_fromserver(buf + 1);
		} else if (strchr(buf, '!') == NULL) {	/* [ŨbZ[W */
			irc_fromuser(buf + 1);
		} else {	/* ̑̃[ŨbZ[W */
			int len;

#ifdef DEBUG
			if (debug)
				printf(buf);
#endif
			sscanf(buf + 1, "%[^!]%*s %s %n", nick, comm, &len);

			buf += len + 1;
			if (*buf == ':') {	/* :hoho!~hoho@10.0.0.1 JOIN :#scot */
				char *p = chan;

				buf++;
				while (*buf && *buf != '\r')
					*p++ = *buf++;
				*p = '\0';
			} else {			/* :hoho!~hoho@10.0.0.1 PRIVMSG #scot :hoe- */
				char *p = rest;
				int len;

				sscanf(buf, " %s %n", chan, &len);
				buf += len;
				while (*buf && *buf != '\r')
					*p++ = *buf++;
				*p = '\0';
			}

			jstr2sjstr(chanbuf, chan);
#ifdef DEBUG
			if (debug) {
				printf("nick [%s]\n", nick);
				printf("comm [%s]\n", comm);
				printf("chan [%s]\n", chanbuf);
				printf("rest [%s]\n", rest);
			}
#endif
			if (!strcmp(comm, "MODE")) {
				printf("%s \033[31mMODE %s %s:%s\033[m\n", getnow(), chanbuf, rest, nick);
			} else if (!strcmp(comm, "JOIN")) {
				printf("%s \033[35m%s has joined %s\033[m\n", getnow(), nick, chanbuf);
				if (!strcmp(irc_nick, nick))
					join_chanlist(chanbuf);
			} else if (!strcmp(comm, "PART")) {
				printf("%s \033[35m%s has parted %s\033[m\n", getnow(), nick, chanbuf);
				if (!strcmp(irc_nick, nick))
					part_chanlist(chanbuf);
			} else if (!strcmp(comm, "QUIT")) {
				printf("%s \033[35m%s has left IRC (%s)\033[m\n", getnow(), nick, chanbuf);
			} else if (!strcmp(comm, "TOPIC")) {
				jstr2sjstr(newbuf, rest + 1);
				printf("%s \033[31mNew topic on %s by %s: %s\033[m\n", getnow(), chanbuf, nick, newbuf);
				set_topic(chanbuf, newbuf);
			} else if (!strcmp(comm, "KICK")) {
				jstr2sjstr(newbuf, rest);
				printf("%s \033[31m%s kicked %s out %s\033[m\n",
					   getnow(), nick, chanbuf, newbuf);
			} else if (!strcmp(comm, "INVITE")) {
				jstr2sjstr(newbuf, rest + 1);
				printf("%s \033[31m%s invites you to %s\033[m\n", getnow(), nick, newbuf);
			} else if (!strcmp(comm, "NICK")) {
				printf("%s \033[31m%s is now %s\033[m\n", getnow(), nick, chan);
			} else if (!strcmp(comm, "NOTICE")) {
				if (!strcmp(irc_nick, chan)) {
					int len;

					sscanf(rest, ":\001%s %n", comm, &len);
#ifdef DEBUG
					if (debug)
						printf("%s\n", comm);
#endif
					if (!strcasecmp(comm, "PING")) {
						time_t ping;

						sscanf(rest + len, " %d", &ping);
						if (ping == ping_code) {
							printf("%s \033[31m%s(%s) = %d second\033[m\n",
								   getnow(), comm, nick, time(NULL) - irc_ping);
							ping_used = FALSE;
#ifdef DEBUG
							if (debug)
								printf("ping_used %d\n", ping_used);
#endif
						} else {
							printf("\033[32mIllegal PING code returns from %s(%d)\033[m\n",
								   nick, ping);
						}
					} else if (!strcasecmp(comm, "TIME")
							   || !strcasecmp(comm, "VERSION")) {
						char *p = newbuf;
						char *q = rest + len;
						char tmpbuf[1024];

						while (*q && *q != '\001')
							*p++ = *q++;
						*p = '\0';
						jstr2sjstr(tmpbuf, newbuf);
						printf("%s \033[31m%s(%s) = %s\033[m\n", getnow(), comm, nick, tmpbuf);
					} else if (!strcasecmp(comm, "USERINFO")
							   || !strcasecmp(comm, "CLIENTINFO")) {
						char *p = newbuf;
						char *q = rest + len + 1;
						char tmpbuf[1024];

						while (*q && *q != '\001')
							*p++ = *q++;
						*p = '\0';
						jstr2sjstr(tmpbuf, newbuf);
						printf("%s \033[31m%s(%s) = %s\033[m\n", getnow(), comm, nick, tmpbuf);
					}
				}
			} else if (!strcmp(comm, "PRIVMSG")) {
				if (*(rest + 1) == '\001') {
					irc_ctcpfrom(sock, nick, rest + 1);
				} else if (!strcmp(chan, irc_nick)) {
					jstr2sjstr(newbuf, rest + 1);
					printf("%s =%s:%s= %s\n", getnow(), chanbuf, nick, newbuf);
				} else {
					jstr2sjstr(newbuf, rest + 1);
					if (irc_keyword && strstr(newbuf, irc_keyword)) {
						if (keyword_bell)
							_iocs_adpcmout(keyword_bell, 0x0403 /* 15.6KHz, stereo */,
										   bell_length);
						else
							putchar('\007');
						if (irc_keyword_bold) {
							printf("%s <%s:%s> \033[37m%s\033[m\n",
								   getnow(), chanbuf, nick, newbuf);
							return TRUE;
						}
					}
					printf("%s <%s:%s> %s\n", getnow(), chanbuf, nick, newbuf);
				}
			} else {
				printf("%s \033[36mUnsupport command %s\033[m", getnow(), comm);
			}
		}
#ifdef DEBUG
	} else {
		if (debug)
			printf("\033[46mWhat? %s\033[m", buf);
#endif
	}
	fflush(stdout);
	return TRUE;
}
