/*
 * kanji.c
 */

__asm("	text\n	dc.b	'$Id: kanji.c,v 1.3 1998/06/07 15:23:56 sharl Exp $'\n	even\n");

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <mbctype.h>
#include "kanji.h"

#define U_BYTE(x)		(unsigned char)((x >> 8) & 0xff)
#define L_BYTE(x)		(unsigned char)(x & 0xff)
#define UL_WORD(x,y)	(unsigned short)((x << 8) | y)

#if 0
static int last_kin;
static unsigned int last_char;
#endif

/****************************************
	convert a char from jis to sjis
****************************************/
static unsigned int
jis2sjis(unsigned int c)
{
	unsigned char u, l;

	u = U_BYTE(c);
	l = L_BYTE(c);

	if (!u)		/* 1bytes */
		return c;
	else if (u & 1) {
		l += 0x1f;
		if (l >= 0x7f)
			l++;
	} else
		l += 0x7e;

	u = (u - ' ' - 1) / 2 + 0x81;
	if (u >= 0xa0)
		u += '@';

	return UL_WORD(u, l);
}

/****************************************
	convert a char from sjis to jis
****************************************/
static unsigned int
sjis2jis(unsigned int c)
{
	unsigned char u, l;

	u = U_BYTE(c);
	l = L_BYTE(c);

	if (!u) {
		if (ismbbkana(l))
			return l & 0x7f;
		else
			return l;
	}
	if (u < 0xa0)
		u -= 0x70;
	else if (u < 0xf0)
		u -= 0xb0;
	else
		return UL_WORD(u, l);
	if (l >= 0x80)
		l--;
	u *= 2;
	if (l >= 0x9e)
		l -= 0x5e;
	else
		u--;
	l -= 0x1f;
	return UL_WORD(u, l);
}

/****************************************
****************************************/
unsigned char *
jstr2sjstr(unsigned char *dst, const unsigned char *src)
{
	unsigned int l, u, c;
	int kin;
	unsigned char *p;
	const unsigned char *q;

	q = src;
	p = dst;
	kin = 0;
	while ((u = *q++) != '\0') {
		if (u == 0x1b) {
			unsigned int s, t;

			s = *q++;
			if (s == '$') {
				t = *q++;
				if (t == '@' || t == 'B')
					kin = 1;
				else {
					q -= 2;
					*p++ = u;
				}
			} else if (s == '(') {
				t = *q++;
				if (t == 'J' || t == 'H' || t == 'B')
					kin = 0;
				else if (t == 'I')
					kin = 2;
				else {
					q -= 2;
					*p++ = u;
				}
			} else if (s == 'K')
				kin = 1;
			else if (s == 'H')
				kin = 0;
			else {
				q -= 1;
				*p++ = u;
			}
			continue;
		}
		if (kin == 1) {
			l = *q++;
			c = jis2sjis(UL_WORD(u, l));
		} else if (kin == 2)
			c = u + 0x80;
		else
			c = u;

		if ((u = U_BYTE(c)))
			*p++ = u;
		*p++ = L_BYTE(c);
	}

	*p++ = '\0';
	return dst;
}

/****************************************
****************************************/
#if 0
unsigned char *
jstr2sjstr2(unsigned char *dst, const unsigned char *src, int *max)
{
	unsigned int l, u, c, i;
	int kin;
	unsigned char *p;
	const unsigned char *q;

	q = src;
	p = dst;
	kin = last_kin;
	last_kin = 0;
	i = 0;

	while (i < *max) {
		if (last_char) {
			u = last_char;
			last_char = 0;
		} else
			u = *q++;
		if (u == 0x1b) {
			unsigned int s, t;

			s = *q++;
			if (s == '$') {
				t = *q++;
				if (t == '@' || t == 'B') {
					kin = 1;
					i += 2;
				} else {
					q -= 2;
					*p++ = u;
				}
			} else if (s == '(') {
				t = *q++;
				if (t == 'J' || t == 'H' || t == 'B') {
					kin = 0;
					i += 2;
				} else if (t == 'I') {
					kin = 2;
					i += 2;
				} else {
					q -= 2;
					*p++ = u;
				}
			} else if (s == 'K')
				kin = 1;
			else if (s == 'H')
				kin = 0;
			else {
				q -= 1;
				*p++ = u;
			}
			i++;
			continue;
		}
		if (kin == 1) {
			if (i + 1 < *max) {
				l = *q++;
				i++;
				c = jis2sjis(UL_WORD(u, l));
			} else {
				last_char = u;
				last_kin = 1;
				c = 0;
			}
		} else if (kin == 2)
			c = u + 0x80;
		else
			c = u;

		if ((u = U_BYTE(c)))
			*p++ = u;
		if ((l = L_BYTE(c)))
			*p++ = l;
		i++;
	}

	if (kin)
		last_kin = kin;

	*p++ = '\0';
	*max = (int)p - (int)dst - 1;
	return dst;
}
#endif

/****************************************
****************************************/
unsigned char *
sjstr2jstr(unsigned char *dst, const unsigned char *src)
{
	unsigned int l, u, c, kin;
	unsigned char *p;
	const unsigned char *q;

	q = src;
	p = dst;
	kin = 0;
	while ((u = *q++) != '\0') {
		if (ismbblead(u)) {
			l = *q++;
			c = sjis2jis(UL_WORD(u, l));

			if (!(kin == 1)) {
				*p++ = 0x1b;
				*p++ = '$';
				*p++ = 'B';
			}
			kin = 1;
		} else if (ismbbkana(u)) {
			if (!(kin == 2)) {
				*p++ = 0x1b;
				*p++ = '(';
				*p++ = 'I';
			}
			kin = 2;
			c = u & 0x7f;
		} else {
			if (kin) {
				*p++ = 0x1b;
				*p++ = '(';
				*p++ = 'B';
			}
			kin = 0;
			c = u;
		}
		if ((u = U_BYTE(c)) != 0)
			*p++ = u;
		*p++ = L_BYTE(c);
	}
	if (kin) {
		*p++ = 0x1b;
		*p++ = '(';
		*p++ = 'B';
	}
	*p++ = '\0';
	return dst;
}

/****************************************
****************************************/
#if 0
unsigned char *
sjstr2jstr2(unsigned char *dst, const unsigned char *src, int *max)
{
	unsigned int l, u, c, kin, i;
	unsigned char *p;
	const unsigned char *q;

	q = src;
	p = dst;
	kin = 0;
	i = 0;
	while (i < *max) {
		if (last_char) {
			u = last_char;
			last_char = 0;
		} else
			u = *q++;
		if (ismbblead(u)) {
			if (i + 1 < *max) {
				l = *q++;
				c = sjis2jis(UL_WORD(u, l));
				i++;
				if (!(kin == 1)) {
					*p++ = 0x1b;
					*p++ = '$';
					*p++ = 'B';
				}
				kin = 1;
			} else {
				last_char = u;
				c = 0;
			}
		} else if (ismbbkana(u)) {
			c = u & 0x7f;
			if (!(kin == 2)) {
				*p++ = 0x1b;
				*p++ = '(';
				*p++ = 'I';
			}
			kin = 2;
		} else {
			if (kin) {
				*p++ = 0x1b;
				*p++ = '(';
				*p++ = 'B';
			}
			kin = 0;
			c = u;
		}
		if ((u = U_BYTE(c)))
			*p++ = u;
		if ((l = L_BYTE(c)))
			*p++ = l;
		i++;
	}
	if (kin) {
		*p++ = 0x1b;
		*p++ = '(';
		*p++ = 'B';
	}
	*p++ = '\0';
	*max = (int)p - (int)dst - 1;
	return dst;
}
#endif
